@@ -72,6 +72,15 @@
 #' To display missing values as is, set the missing parameter to NULL.  To
 #' replace missing values with a character string (such as ".", "-", or "<NA>")
 #' pass the desired character string to the missing parameter.
+#' @param font The font to use on the report.  The font specified will be
+#' used for the entire report.  Valid values are "Courier", "Arial", "Times",
+#' and "fixed".  The value of "fixed" will create a fixed-width, text style
+#' report in Courier font.  The \code{font} parameter only applies to RTF reports
+#' at this time.  The default value is "fixed".
+#' @param font_size The size of the font to use on the report. The \code{font_size}
+#' specified will be used for the entire report.  Valid values are 8, 10, 
+#' and 12.  The \code{font_size} parameter only applies to RTF and PDF output types.
+#' The default value is 10.
 #' @return A new report_spec object.
 #' @family report
 #' @seealso \code{\link{create_table}}, \code{\link{create_text}}, and
@@ -131,10 +140,10 @@
 #' @export
 create_report <- function(file_path = "", output_type = "TXT", 
                           orientation ="landscape", units = "inches",
-                          paper_size = "letter", missing = "") {#,
-                         # font = "fixed", font_size = NULL) {
-  font = "fixed"
-  font_size = NULL
+                          paper_size = "letter", missing = "",
+                          font = "fixed", font_size = NULL) {
+  # font = "fixed"
+  # font_size = NULL
 
   x <- structure(list(), class = c("report_spec", "list"))
 
@@ -215,7 +224,7 @@ create_report <- function(file_path = "", output_type = "TXT",
   if (output_type %in% c("TXT", "PDF", "RTF")) {
     
     # Set default options for text
-    # This sets line_height and char_width
+    # This sets row_height and char_width
     # which are needed for all conversions from 
     # units to text
     x <- options_fixed(x)
@@ -658,7 +667,7 @@ options_fixed <- function(x, editor = NULL, cpuom = NULL, lpuom = NULL,
 
   x$font_size <- font_size
   x$char_width <- 1 / x$cpuom
-  x$line_height <- 1 / x$lpuom
+  x$row_height <- 1 / x$lpuom
   x$user_line_size <- line_size
   x$user_line_count <- line_count
   
@@ -1002,7 +1011,8 @@ page_header <- function(x, left="", right="", blank_row = "none"){
 #' \code{blank_row} parameter.
 #'
 #' Each title string must fit within the available space.  The \strong{reporter}
-#' package will not wrap titles.  If a title does not fit within the 
+#' package will not wrap titles on fixed-width reports.  
+#' If a title does not fit within the 
 #' available space, an error will be generated.  In these situations, either
 #' shorten the title or split it into multiple titles that each fit within the
 #' available space. 
@@ -1015,7 +1025,22 @@ page_header <- function(x, left="", right="", blank_row = "none"){
 #' @param blank_row Where to place a blank row.  Valid values are 'above',
 #' 'below', 'both', or 'none'.  Default is 'below'.
 #' @param borders Whether and where to place a border. Valid values are 'top',
-#' 'bottom', 'all', or 'none'.  Default is 'none'.  
+#' 'bottom', 'left', 'right', 'outside', 'inside', 'all', or 'none'.  
+#' Default is 'none'.   The 
+#' 'left', 'right', 'outside', and 'inside' border specifications only 
+#' apply to RTF reports.
+#' @param width The width of the title header.  If the title header is attached
+#' to the report, valid values are 'page' or a numeric width, and the default
+#' is 'page'. If the title header is attached to the
+#' table, plot, or text content, the valid values are 'page', 'content' or a 
+#' numeric value, and the default is 'content'.  The value 'content' means the 
+#' footnotes will be aligned to the width of the table, plot, or text
+#' content.  The value
+#' 'page' means the footnotes will be aligned to the width of the page. 
+#' In addition to these two convenience settings, you 
+#' may also specify a specific width in the current units of measure.  The
+#' units of measure is determined by the 'units' parameter on 
+#' \code{\link{create_report}}.
 #' @return The modified report.
 #' @family report
 #' @examples
@@ -1066,7 +1091,7 @@ page_header <- function(x, left="", right="", blank_row = "none"){
 #' #     * In billions of dollars
 #' @export
 title_header <- function(x, ..., right = "", 
-                         blank_row = "below", borders = "none") {
+                         blank_row = "below", borders = "none", width = NULL) {
   
 
   diff <- setdiff(class(x), c("list"))
@@ -1096,15 +1121,35 @@ title_header <- function(x, ..., right = "",
     stop(paste("Blank row parameter invalid.  Valid values are", 
                "'above', 'below', 'both', or 'none'."))
   
-  if (!all(borders %in% c("top", "bottom", "all", "none")))
+  if (!all(borders %in% c("top", "bottom", "left", "right", 
+                          "outside", "inside", "all", "none"))) {
     stop(paste("Borders parameter invalid.  Valid values are", 
-               "'top', 'bottom', 'all', or 'none'."))
+      "'top', 'bottom', 'left', 'right', 'outside', 'inside, 'all', or 'none'."))
+  }
+  
+  if (is.null(width)) {
+    if (any(class(x) %in% c("report_spec"))) 
+      width <- "page"
+    else 
+      width <- "content"
+  } else {
+    if (any(class(x) %in% c("report_spec"))) {
+      if (!width %in% c("page") & !is.numeric(width))
+        stop("Width parameter invalid.  Valid values are 'page' or a number.")
+    } else {
+      if (!width %in% c("page", "content") & !is.numeric(width))
+        stop(paste("Width parameter invalid.  Valid values are 'page',",
+                   "'content' or a number."))
+    }
+    
+  }
   
   # Assign attributes
   ttl_hdr$titles <-  c(...)
   ttl_hdr$blank_row <- blank_row
   ttl_hdr$borders <- borders
   ttl_hdr$right <- right
+  ttl_hdr$width <- width
   
   x$title_hdr[[length(x$title_hdr) + 1]] <- ttl_hdr
   
@@ -1149,7 +1194,8 @@ title_header <- function(x, ..., right = "",
 #' \code{blank_row} parameter.
 #' 
 #' Each title string must fit within the available space.  The \strong{reporter}
-#' package will not wrap titles.  If a title does not fit within the 
+#' package will not wrap titles on fixed-width reports.  
+#' If a title does not fit within the 
 #' available space, a warning will be generated and the title will be 
 #' truncated.  In these situations, either
 #' shorten the title or split it into multiple titles that each fit within the
@@ -1163,7 +1209,22 @@ title_header <- function(x, ..., right = "",
 #' @param blank_row Where to place a blank row.  Valid values are 'above',
 #' 'below', 'both', or 'none'.  Default is "below".
 #' @param borders Whether and where to place a border. Valid values are 'top',
-#' 'bottom', 'all', or 'none'.  Default is "none".
+#' 'bottom', 'left', 'right', 'outside', 'inside', 'all', or 'none'.  
+#' Default is "none".  
+#' The 'left', 'right', 'outside', and 'inside' border specifications 
+#' only apply to RTF reports.
+#' @param width The width of the titles block.  If the titles are attached
+#' to the report, valid values are 'page' or a numeric width, and the default
+#' is 'page'. If the titles are attached to the
+#' table, plot, or text content, the valid values are 'page', 'content' or a 
+#' numeric value, and the default is 'content'.  The value 'content' means the 
+#' titles will be aligned to the width of the table, plot, or text
+#' content.  The value
+#' 'page' means the titles will be aligned to the width of the page. 
+#' In addition to these two convenience settings, you 
+#' may also specify a specific width in the current units of measure.  The
+#' units of measure is determined by the 'units' parameter on 
+#' \code{\link{create_report}}.
 #' @return The modified report.
 #' @family report
 #' @examples
@@ -1212,7 +1273,7 @@ title_header <- function(x, ..., right = "",
 #' #     * In billions of dollars
 #' @export
 titles <- function(x, ..., align = "center", blank_row = "below", 
-                   borders = "none"){
+                   borders = "none", width = NULL){
 
   # Create title structure
   ttl <- structure(list(), class = c("title_spec", "list"))
@@ -1227,15 +1288,38 @@ titles <- function(x, ..., align = "center", blank_row = "below",
     stop(paste("Blank row parameter invalid.  Valid values are", 
                "'above', 'below', 'both', or 'none'."))
   
-  if (!all(borders %in% c("top", "bottom", "all", "none")))
+  if (!all(borders %in% c("top", "bottom", "left", "right", "all", "outside", 
+                          "inside", "none")))
     stop(paste("Borders parameter invalid.  Valid values are", 
-               "'top', 'bottom', 'all', or 'none'."))
+      "'top', 'bottom', 'left', 'right', 'outside', 'inside', 'all', or 'none'."))
 
+  
+
+  if (is.null(width)) {
+    if (any(class(x) %in% c("report_spec"))) 
+      width <- "page"
+    else 
+      width <- "content"
+    
+  } else {
+    if (any(class(x) %in% c("report_spec"))) {
+      if (!width %in% c("content") & !is.numeric(width))
+        stop("Width parameter invalid.  Valid values are 'page' or a number.")
+    } else {
+      if (!width %in% c("page", "content") & !is.numeric(width))
+        stop(paste("Width parameter invalid.  Valid values are 'page',",
+                   "'content' or a number."))
+    }
+    
+  }
+      
+  
   # Assign attributes
   ttl$titles <-  c(...)
   ttl$blank_row <- blank_row
   ttl$borders <- borders
   ttl$align <- align
+  ttl$width <- width
   
 
   x$titles[[length(x$titles) + 1]] <- ttl
@@ -1271,7 +1355,8 @@ titles <- function(x, ..., align = "center", blank_row = "below",
 #' \code{blank_row} parameter.
 #' 
 #' Each footnote string must fit within the available space.  The \strong{reporter}
-#' package will not wrap footnotes.  If a footnote does not fit within the 
+#' package will not wrap footnotes on fixed-width reports.  
+#' If a footnote does not fit within the 
 #' available space, a warning will be generated and the footnote will be 
 #' truncated.  In these situations, either
 #' shorten the footnote or split it into multiple footnotes that each fit within 
@@ -1284,13 +1369,28 @@ titles <- function(x, ..., align = "center", blank_row = "below",
 #' @param blank_row Whether to print a blank row above or below the footnote.
 #' Valid values are 'above', 'below', 'both', or 'none'.  Default is 'above'.
 #' @param borders Whether to print a border above or below the footnote. Valid
-#' values are 'top', 'bottom', 'all',  or 'none'.  Default is 'none'.  
+#' values are 'top', 'bottom', 'outside', 'inside', 'all',  or 'none'.  
+#' Default is 'none'.  
 #' For fixed width reports, the 
 #' border character will be taken from the value of the \code{uchar} parameter
-#' on the \code{\link{options_fixed}} function.
+#' on the \code{\link{options_fixed}} function.  The 
+#' 'left', 'right', 'outside', and 'inside' border specifications only apply 
+#' to RTF reports.
 #' @param valign The vertical position to align the footnotes.  Valid
 #' values are: 'top' and 'bottom'.  For footnotes attached to a report,
 #' default is 'bottom'.  For footnotes attached to content, default is 'top'.
+#' @param width The width of the footnotes block.  If the footnotes are attached
+#' to the report, valid values are 'page' or a numeric width, and the default
+#' is 'page'. If the footnotes are attached to the
+#' table, plot, or text content, the valid values are 'page', 'content' or a 
+#' numeric value, and the default is 'content'.  The value 'content' means the 
+#' footnotes will be aligned to the width of the table, plot, or text
+#' content.  The value
+#' 'page' means the footnotes will be aligned to the width of the page. 
+#' In addition to these two convenience settings, you 
+#' may also specify a specific width in the current unit of measure.  The
+#' unit of measure is determined by the 'units' parameter on 
+#' \code{\link{create_report}}.
 #' @return The modified report.
 #' @family report
 #' @examples
@@ -1340,9 +1440,7 @@ titles <- function(x, ..., align = "center", blank_row = "below",
 #' #     * In billions of dollars
 #' @export
 footnotes <- function(x, ..., align = "left", blank_row = "above", 
-                      borders = "none"
-                      , valign = NULL
-                      ){
+                      borders = "none", valign = NULL, width = NULL){
 
   # Create footnote structure
   ftn <- structure(list(), class = c("footnote_spec", "list"))
@@ -1365,21 +1463,37 @@ footnotes <- function(x, ..., align = "left", blank_row = "above",
     stop(paste("Blank row parameter invalid.  Valid values are", 
                "'above', 'below', 'both', or 'none'."))
   
-  if (any(!borders %in% c("top", "bottom", "all", "none")))
+  if (any(!borders %in% c("top", "bottom", "left", "right", "all", 
+                          "outside", "inside", "none")))
     stop(paste("Borders parameter invalid.  Valid values are", 
-               "'top', 'bottom', 'all', or 'none'."))
+      "'top', 'bottom', 'left', 'right', 'outside', 'inside', 'all', or 'none'."))
+  
+  if (is.null(width)) {
+    if (any(class(x) %in% c("report_spec"))) 
+      width <- "page"
+    else {
+      width <- "content"
+      if (!is.null(valign))
+        if (valign == "bottom")
+          width <- "page"
+    }
+  } else {
+    if (any(class(x) %in% c("report_spec"))) {
+      if (!width %in% c("content") & !is.numeric(width))
+        stop("Width parameter invalid.  Valid values are 'page' or a number.")
+    } else {
+      if (!width %in% c("page", "content") & !is.numeric(width))
+        stop(paste("Width parameter invalid.  Valid values are 'page',",
+                   "'content' or a number."))
+    }
+  }
 
   ftn$footnotes <- ft
   ftn$blank_row <- blank_row
   ftn$align <- align
   ftn$borders <- borders
-  
-  if ("report_spec" %in% class(x)) {
-    ftn$container <- "report" 
-  } else {
-    ftn$container <- "content" 
-  }
-  
+  ftn$width <- width  
+
   if (is.null(valign)) {
     if ("report_spec" %in% class(x))
       ftn$valign <- "bottom"
@@ -1526,7 +1640,7 @@ page_footer <- function(x, left="",  center="", right="", blank_row = "above"){
 #' left, right, or center. Use the \code{align} parameter to specify the 
 #' alignment. 
 #' 
-#' You must be sort the data by the page by variable prior to reporting.
+#' You must sort the data by the page by variable prior to reporting.
 #' The page by labels will appear in the sorted order.  Failure to sort 
 #' the page by variable prior to reporting may produce unexpected results.
 #' 
@@ -1542,6 +1656,9 @@ page_footer <- function(x, left="",  center="", right="", blank_row = "above"){
 #' @param blank_row Indicates whether a blank row is desired above or below
 #' the page by.  Default value is 'none'.  Valid values are 'above', 'below',
 #' 'both', or 'none'.
+#' @param borders Whether and where to place a border. Valid values are 'top',
+#' 'bottom', 'left', 'right', 'all', 'outside', or 'none'.  Default is "none".  
+#' The 'left' and 'right' border specifications only apply to RTF reports.
 #' @family report
 #' @seealso \code{\link{create_table}} to create a table, and 
 #' \code{\link{create_plot}} to create a plot.  
@@ -1658,7 +1775,7 @@ page_footer <- function(x, left="",  center="", right="", blank_row = "above"){
 #' # 2020-10-25 19:33:35                                                Page 3 of 3 
 #' @export
 page_by <- function(x, var, label = NULL, align = "left",
-                    blank_row = "below") {
+                    blank_row = "below", borders = "none") {
   
   
   # Create page by structure
@@ -1674,6 +1791,11 @@ page_by <- function(x, var, label = NULL, align = "left",
                "'both', or 'none'."))
   }
   
+  
+  if (!all(borders %in% c("top", "bottom", "left", "right", "all", "outside", "none")))
+    stop(paste("Borders parameter invalid.  Valid values are", 
+               "'top', 'bottom', 'left', 'right', 'all', 'outside', or 'none'."))
+  
   var_c <- as.character(substitute(var, env = environment()))
   
   pb$var <- var_c
@@ -1684,6 +1806,7 @@ page_by <- function(x, var, label = NULL, align = "left",
   
   pb$align <- align
   pb$blank_row <- blank_row
+  pb$borders <- borders
 
   x$page_by <- pb
   
