@@ -840,3 +840,212 @@ test_that("rtf26: line_size and line_count overrides work as expected.", {
   
 })
 
+
+test_that("rtf27: Plot Borders work as expected.", {
+  
+  library(ggplot2)
+  
+  fp <- file.path(base_path, "rtf/test27.rtf")
+  
+  p <- ggplot(mtcars, aes(x=cyl, y=mpg)) + geom_point()
+  
+  
+  plt <- create_plot(p, height = 4, width = 8, borders = "all") %>% 
+    titles("My plot", borders = "none") %>% 
+    footnotes("My plot footnotes", borders = "none")
+  
+  rpt <- create_report(fp, output_type = "RTF") %>%
+    page_header("Client", "Study: XYZ") %>%
+    set_margins(top = 1, bottom = 1) %>%
+    add_content(plt, align = "right") %>%
+    page_footer("Time", "Confidential", "Page [pg] of [tpg]")
+  
+  
+  res <- write_report(rpt)
+  
+  #print(res)
+  
+  expect_equal(file.exists(fp), TRUE)
+  expect_equal(res$pages, 1)
+})
+
+test_that("rtf28: Table Borders that spans multiple pages work as expected.", {
+  
+  fp <- file.path(base_path, "rtf/test28.rtf")
+  
+  rpt <- create_report(fp, output_type = "RTF") %>%
+    titles("IRIS Data Frame") %>%
+    add_content(create_table(iris, borders = "all")) %>% 
+    footnotes("Here is a footnote")
+  
+  
+  res <- write_report(rpt)
+  
+  expect_equal(file.exists(fp), TRUE)
+  expect_equal(res$pages, 4)
+  #write_registration_file(file.path(base_path,"./rtf/reg.txt"))
+})
+
+test_that("rtf29: Simplest RTF Plot with valign top works as expected.", {
+  
+  library(ggplot2)
+  
+  fp <- file.path(base_path, "rtf/test29.rtf")
+  
+  p <- ggplot(mtcars, aes(x=cyl, y=mpg)) + geom_point()
+  
+  plt <- create_plot(p, height = 4, width = 8)
+  
+  
+  rpt <- create_report(fp, output_type = "RTF") %>%
+    page_header("Client", "Study: XYZ") %>%
+    titles("Figure 1.0", "MTCARS Miles per Cylinder Plot") %>%
+    set_margins(top = 1, bottom = 1) %>%
+    add_content(plt, align = "center") %>%
+    footnotes("* Motor Trend, 1974", valign = "top") %>%
+    page_footer("Time", "Confidential", "Page [pg] of [tpg]")
+  
+  
+  res <- write_report(rpt)
+  
+  #print(res)
+  
+  expect_equal(file.exists(fp), TRUE)
+  expect_equal(res$pages, 1)
+  
+  
+})
+
+test_that("rtf30: Simplest RTF Plot with valign bottom works as expected.", {
+  
+  library(ggplot2)
+  
+  fp <- file.path(base_path, "rtf/test30.rtf")
+  
+  p <- ggplot(mtcars, aes(x=cyl, y=mpg)) + geom_point()
+  
+  plt <- create_plot(p, height = 4, width = 8) %>%
+    footnotes("* Motor Trend, 1974", valign = "bottom")
+  
+  
+  rpt <- create_report(fp, output_type = "RTF") %>%
+    page_header("Client", "Study: XYZ") %>%
+    titles("Figure 1.0", "MTCARS Miles per Cylinder Plot") %>%
+    set_margins(top = 1, bottom = 1) %>%
+    add_content(plt, align = "center") %>%
+    page_footer("Time", "Confidential", "Page [pg] of [tpg]")
+  
+  
+  res <- write_report(rpt)
+  
+  #print(res)
+  
+  expect_equal(file.exists(fp), TRUE)
+  expect_equal(res$pages, 1)
+  
+  
+})
+
+test_that("rtf31: Simplest RTF Text with valign top works as expected.", {
+  
+
+  fp <- file.path(base_path, "rtf/test31.rtf")
+
+  txt <- create_text(cnt, width = 6)
+  
+  
+  rpt <- create_report(fp, output_type = "RTF") %>%
+    page_header("Client", "Study: XYZ") %>%
+    titles("Text 1.0", "MTCARS Miles per Cylinder Text") %>%
+    set_margins(top = 1, bottom = 1) %>%
+    add_content(txt, align = "center") %>%
+    footnotes("* Motor Trend, 1974", valign = "top") %>%
+    page_footer("Time", "Confidential", "Page [pg] of [tpg]")
+  
+  
+  res <- write_report(rpt)
+  
+  #print(res)
+  
+  expect_equal(file.exists(fp), TRUE)
+  expect_equal(res$pages, 1)
+  
+  
+})
+
+test_that("rtf32: Simplest RTF Text with valign bottom works as expected.", {
+  
+
+  fp <- file.path(base_path, "rtf/test32.rtf")
+  
+  txt <- create_text(cnt, width = 6) %>%
+    footnotes("* Motor Trend, 1974", valign = "bottom")
+  
+  rpt <- create_report(fp, output_type = "RTF") %>%
+    page_header("Client", "Study: XYZ") %>%
+    titles("Figure 1.0", "MTCARS Miles per Cylinder Plot") %>%
+    set_margins(top = 1, bottom = 1) %>%
+    add_content(txt, align = "center") %>%
+    page_footer("Time", "Confidential", "Page [pg] of [tpg]")
+  
+  
+  res <- write_report(rpt)
+  
+  #print(res)
+  
+  expect_equal(file.exists(fp), TRUE)
+  expect_equal(res$pages, 1)
+  
+  
+})
+
+
+test_that("rtf33: use_attributes parameter table works as expected.", {
+  
+  
+  fp1 <- file.path(base_path, "rtf/test33a.rtf")
+  fp2 <- file.path(base_path, "rtf/test33b.rtf")
+  fp3 <- file.path(base_path, "rtf/test33c.rtf")
+  
+  
+  dat <- mtcars[1:10, ]
+  attr(dat$mpg, "label") <- "Miles per gallon"
+  attr(dat$cyl, "format") <- "%.1f"
+  attr(dat$hp, "width") <- 2
+  fattr(dat$vs) <- list(width = 2, justify = "center")
+  
+  tbl <- create_table(dat) 
+  
+  # Test default 
+  rpt <- create_report(fp1, output_type = "RTF", font = "fixed") %>% 
+    add_content(tbl)
+  
+  res <- write_report(rpt)
+  
+  expect_equal(file.exists(fp1), TRUE)
+  expect_equal(res$pages, 1)
+  
+  # Test none
+  tbl <- create_table(dat, use_attributes = "none") 
+  
+  rpt <- create_report(fp2, output_type = "RTF", font = "fixed") %>% 
+    add_content(tbl)
+  
+  res <- write_report(rpt)
+  
+  expect_equal(file.exists(fp2), TRUE)
+  
+  
+  # Test some
+  tbl <- create_table(dat, use_attributes = c("format", "width")) 
+  
+  rpt <- create_report(fp3, output_type = "RTF", font = "fixed") %>% 
+    add_content(tbl)
+  
+  res <- write_report(rpt)
+  
+  expect_equal(file.exists(fp3), TRUE)
+  expect_equal(res$pages, 1)
+  
+})
+
